#!/bin/bash

### 
#
# Script for finding orphan folders, which no-one needs anymore
#
# usage:
#
# ./check_var_scratch.sh [user/group]
#
# if $1 not set, it looks for dirs of $USER
#
###


set -o errexit
set -o verbose

###
# setting var for whom or which group we're looking for
###
if [ -z "$1" ]; then
  WHO=$USER
else
  WHO=$1
fi

### generate tmp file for /var/scratch dirs
### iit is not reachable atm
qhost | grep -v iit.internal | awk '/internal/{print $1}' | while read host; do echo $host; ssh -n $host "ls -lah /var/scratch"; done > /tmp/tst.txt

### convert to list of shame
awk '/\.internal/{host=$1;next} NF<=2{print host,$0} NF>2{print $0,host}' /tmp/tst.txt > /tmp/listOfShame.txt

### number of dirs per user
echo User Shame
awk 'NF>3{print $3}' /tmp/listOfShame.txt | sort | uniq -c
### number of dirs per group
echo Group Shame
awk 'NF>3{print $4}' /tmp/listOfShame.txt | sort | uniq -c

### awk thing for listing every dir of $WHO ($1 or $USER)
grep "$WHO" /tmp/listOfShame.txt | \
  awk '
    {
      print $NF; 
      system("ssh -n "$NF" \"ls -lah /var/scratch/"$(NF-1)"\""); 
      printf "delete? (y/N) "; getline answer < "/dev/tty"; 
      if(answer ~/^(y|Y)(es)?$/){
        system("ssh -n "$NF" \"rm -rf /var/scratch/"$(NF-1)"\""); 
        print "deleted"
      } else {
      print "continue without deleting"
      }
    }'

